/*  
	Copyright (C) 2007 Salvatore Scarciglia <laas.mono@gmail.com>
	http://laas.altervista.org

    	This code is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.
    	This code is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
    	You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/
using System;
using System.IO;
using System.Diagnostics;
using Gtk;
using Gdk;
using Glade;
using System.Xml;
using System.Collections;
using System.Threading;


public class ckaMain { // Main Class containing the Main function 
	[Widget] Gtk.Window 			mainWindow; // the main window of the application
	[Widget] Gtk.TreeView 			fileSystemViewer; // the file system viewer 
	[Widget] Gtk.TreeView 			thumbsViewer; // the thumbs viewer
	[Widget] Gtk.TreeView 			treeBookmark; // the bookmark viewer
	[Widget] Gtk.Image				mainImageViewer; // the container of the image to be displayed in the "view" ara
	[Widget] Gtk.Label				lblStatus1; // a label inside the status bar used to provide informations about current image
	[Widget] Gtk.VPaned				vpaned1; // the Panel containing the thumbsViewer and the BookmarkViewer 
	[Widget] Gtk.Toolbar			tools; // the tool bar
	// bottom toolbar ToolButton
	[Widget] Gtk.ToolButton			cmdFirstImg; // select the first item of the list
	[Widget] Gtk.ToolButton			cmdPreviousImg; // select the previous item of the list
	[Widget] Gtk.ToolButton			cmdNextImg; // sleect the next item of the list
	[Widget] Gtk.ToolButton			cmdLastImg; // select the last item of the list
	[Widget] Gtk.ToolButton			cmdOriginalSize; // display current image in its original size
	[Widget] Gtk.ToolButton			cmdFitToWin; // fit current image to the windows
	[Widget] Gtk.ToolButton			cmdBackground; // set image as backround (works on Gnome)
	[Widget] Gtk.ToolButton			cmdZoomIn; // Zoom in image
	[Widget] Gtk.ToolButton			cmdZoomOut; // Zoom out image
	[Widget] Gtk.ToolButton			cmdQuit; // exit Cooka
	[Widget] Gtk.ToolButton			cmdHome; // get my home directory
	[Widget] Gtk.ToolButton			cmdAbout; // display the About window
	[Widget] Gtk.ToolButton			cmdRotateRight; // rotate image to the right
	[Widget] Gtk.ToolButton			cmdRotateLeft; // rotate image to the left
	// Bookmark handler
	[Widget] Gtk.Button				cmdAdd; // add selected folder to the list of bookmark
	[Widget] Gtk.Button				cmdDel; // remove selected bookmark from the list
	[Widget] Gtk.Notebook			fileSystemBook; // the notebook containing thumbsViewer and bookmark viewer
	[Widget] Gtk.Viewport			imgViewport; // 
	[Widget] Gtk.CheckButton		mnuOptionsFit; // Always fit image to window
	
	private int	thumbsHeight = 48; // thumbs height
	private int	thumbsWeight = 60; // thumbs width
	private int	typeOfView = 0;  // defines how image will be displayed (original size or fit to win) 
	private bool showHidden = false; // do not show hidden folder
	private bool imageRotated = false; // current image has being rotated
	private bool imageIsSelected = false; // an image was selected
	private string	selectedFolder = ""; // selected folder
	private string	selectedBookmark = ""; // selected bookmark
	private string 	supportedFormat =  "#png#jpeg#jpg#gif#tga#bmp#ico#tiff#tif#ani#xpm#bmp#pnm#pbm#pgm#ppm#xbm#tga#svg#";
	private string	folderToRead = ""; 
	private double	zoomFactor  = 10.0; // how to zoom in  and out images
	private string 	ckaVersion = "0.4"; // current application version
	private int	zoomed = 0; // zoom ratio
	private int rotated = 0; // rotate action
	private int howManyImages = 0; // how many images in current folder (supported format)
	private bool fullscreen = false; // view in fullscreen mode
	private bool maximized = false; // maximized mode
	private ListStore fileSystemStore; // to handle file system TreeVIew
	private ListStore thumbsStore; //  to handle Thumbs TreeVIew
	private ListStore bookmarkStore;  // to handle Bookmark TreeVIew
	private ckaFileSystemReader fsReader; // object used to read current selected folder
	private Pixbuf startUpImage; // the image to be displayed when Cooka start 
	private ckaImage selectedImage; // current selected image
	private settingReader userSettingReader; // the settings reader

	public static void Main (string[] args){ // the Main function (all starts here)
		new ckaMain (args);
	}

	public ckaMain (string[] args) { // main constructor
		Application.Init ();
		Glade.XML gxml = new Glade.XML (null, "gui.glade", "mainWindow", null); // connect to the glade resource
		gxml.Autoconnect (this);
		go(); // build application
		Application.Run (); // run application
	}
		
	private void go(){ // Cooka setup
		// defines event handler for each "active" items of the GUI
		cmdBackground.Clicked+=on_cmdBackground_clicked;
		cmdZoomIn.Clicked+=on_cmdZoomIn_clicked;
		cmdZoomOut.Clicked+=on_cmdZoomOut_clicked;
		cmdFitToWin.Clicked+=on_cmdFitToWin_clicked;
		cmdOriginalSize.Clicked+=on_cmdOriginalSize_clicked;
		mnuOptionsFit.Clicked+=on_mnuOptions_aFit_activate;
		cmdQuit.Clicked+=on_mnuFile_quit_activate;
		cmdAbout.Clicked+=on_mnuHelp_About_activate;
		cmdHome.Clicked+=on_tbarGoHome_clicked;
		cmdRotateRight.Clicked+=on_cmdRotateRight_Clicked;
		cmdRotateLeft.Clicked+=on_cmdRotateLeft_Clicked;
		cmdAdd.Clicked+=on_cmdAdd_clicked;
		cmdDel.Clicked+=on_cmdDel_clicked;
		cmdFirstImg.Clicked+=onFirstImage;
		cmdPreviousImg.Clicked+=onPreviousImage;
		cmdNextImg.Clicked+=onNextImage;
		cmdLastImg.Clicked+=onLastImage;
		
		// Setup every TreeView
		setupFileSystemTreeView();
		setupThumbsViewer(); 
		setupBookmarkViewer();
		
		populateFileSystemViewer(); // read home folder and populates the file system treeview
		userSettingReader = new settingReader(bookmarkStore); // read user's settings in order to fill bookmark treeview
		startUpImage = new Pixbuf(null, "startup.png"); // get the startup image (Cooka logo)
		mainImageViewer.FromPixbuf = startUpImage;  
		userSettingReader.readAndPopulateBookmark();
		mainWindow.Icon = new Pixbuf(null, "logo_v1_ico.png"); // set the main window's icon... 
		mainWindow.Title = "Cooka - release "+ckaVersion; // ...and the title
		mainWindow.KeyReleaseEvent +=keyHandler; // key event handler --> THIS IS VERY IMPORTANT
		mainWindow.DeleteEvent+=OnWindowDeleteEvent; // handle the window's closing event
		pushOnVirtualStatusBar(System.Environment.GetEnvironmentVariable("HOME")); // put current dir in the status bar label
	}	

	private void populateFileSystemViewer(){ // call the File System Reader object and fill the related TreeView
		fsReader = new ckaFileSystemReader(System.Environment.GetEnvironmentVariable("HOME"),fileSystemStore,showHidden);
		fsReader.refresh();
	}
	
	private void setupFileSystemTreeView(){ // Set up file system Tree vIew
		TreeViewColumn col = new TreeViewColumn();
		CellRendererPixbuf pixr = new CellRendererPixbuf();
		col.PackStart(pixr, false);
		col.AddAttribute(pixr, "pixbuf", 1); // It will display an icon representing a "folder" ...
		
		CellRendererText colr = new CellRendererText ();
		col.PackStart (colr, true);
		col.AddAttribute (colr, "text", 0); // ...and the name of the folder
		fileSystemViewer.AppendColumn (col);
				
		col = new TreeViewColumn ();
		colr = new CellRendererText ();
		colr.Editable = false;
		col.PackStart (colr, true);
		col.AddAttribute (colr, "text", 0);
		col.Visible = false;
		fileSystemViewer.AppendColumn (col);				
		
		fileSystemViewer.RowActivated+=on_fileSystemViewer_row_activated; // handle duoble click on treeview's rows
		fileSystemStore = new ListStore (typeof (string),typeof (Gdk.Pixbuf), typeof (string));
		fileSystemViewer.Model = fileSystemStore;
	}
	
	
	private void setupBookmarkViewer(){ // Set uo the bookmark treeview
		TreeViewColumn col = new TreeViewColumn();
		CellRendererText colr = new CellRendererText ();
		col.PackStart (colr, true);
		col.AddAttribute (colr, "markup", 0); // it will use just a text item representing the path to the folder
		treeBookmark.RowActivated+=on_treeBookmark_row_activated;
		treeBookmark.AppendColumn (col);
		bookmarkStore = new ListStore (typeof (string));
		treeBookmark.Model = bookmarkStore;		
	}	
	
	private void  on_fileSystemViewer_row_activated(object o, EventArgs args){ // double click folder
		TreeIter iter = new TreeIter();
		fileSystemStore.GetIterFirst(out iter);
		if (fileSystemViewer.Selection.IterIsSelected(iter)) {
			selectedFolder = (string)fileSystemStore.GetValue(iter, 2);
		}		
		while(fileSystemStore.IterNext(ref iter)) {
			if(fileSystemViewer.Selection.IterIsSelected(iter)) {
				selectedFolder = (string)fileSystemStore.GetValue(iter, 2);
			}
		}	
		cmdAdd.Sensitive = true;
    	fsReader.currentPath = selectedFolder;
    	fsReader.refresh();
    	folderToRead = selectedFolder;
    	pushOnVirtualStatusBar(selectedFolder);
        readDirAndPopulateArray();		
	}
	
	private void  on_treeBookmark_row_activated(object o, EventArgs args){ // double click bookmark
		TreeIter iter = new TreeIter();
		bookmarkStore.GetIterFirst(out iter);
		if (treeBookmark.Selection.IterIsSelected(iter)) {
			selectedBookmark = (string)bookmarkStore.GetValue(iter, 0);
		}		
		while(bookmarkStore.IterNext(ref iter)) {
			if(treeBookmark.Selection.IterIsSelected(iter)) {
				selectedBookmark = (string)bookmarkStore.GetValue(iter, 0);
			}
		}		
		cmdDel.Sensitive = true;
    		fsReader.currentPath = selectedBookmark;
    		fsReader.refresh();
    		folderToRead = selectedBookmark;
    		pushOnVirtualStatusBar(selectedBookmark);
        	readDirAndPopulateArray();
	}	
	
	private void on_tbarGoHome_clicked(object sender, EventArgs e){	// click on HOME button on Toolbar
    		fsReader.currentPath = System.Environment.GetEnvironmentVariable("HOME");
    		fsReader.refresh();
    		pushOnVirtualStatusBar(System.Environment.GetEnvironmentVariable("HOME"));
    		folderToRead = System.Environment.GetEnvironmentVariable("HOME");
    		pushOnVirtualStatusBar(folderToRead);
        	readDirAndPopulateArray();    		
  	}
	
	private void browseImage(int type){ // used to select a specific row of the thumbs treeview without clicking on it
		TreeIter x;
		TreeModel y;
		thumbsViewer.Selection.GetSelected(out y, out x);
		TreePath p;
		switch(type) {
			case 1: // First image
				p = new TreePath("0");
				thumbsViewer.Selection.SelectPath(p);
				break;
			case 2: // Previous image
				p = y.GetPath(x);
				p.Prev();
				thumbsViewer.Selection.SelectPath(p);
				break;
			case 3: // Next image
				p = y.GetPath(x);
				p.Next();
				thumbsViewer.Selection.SelectPath(p);
				break;
			case 4: // Last image
				p = new TreePath(""+(howManyImages-1));
				thumbsViewer.Selection.SelectPath(p);			
				break;
		}
	}
	
	// event handlers of the navigation buttons 
	private void onFirstImage(object sender, EventArgs e){ browseImage(1); }
	private void onPreviousImage(object sender, EventArgs e){ browseImage(2);}
	private void onNextImage(object sender, EventArgs e){browseImage(3);}
	private void onLastImage(object sender, EventArgs e){browseImage(4);}
	
	private void on_cmdOriginalSize_clicked(object sender, EventArgs e){ // display image in its original size
		mainImageViewer.FromPixbuf = getImage(selectedImage, 1);
		zoomed=0;
	}

	private void on_cmdFitToWin_clicked(object sender, EventArgs e){ // fit image to window
		mainImageViewer.FromPixbuf = getImage(selectedImage, 2);
		zoomed=0;
	}
	
	private void on_cmdAdd_clicked(object sender, EventArgs e){ // add selected folder to bookmark list
			userSettingReader.addBookmark(selectedFolder);
	}	
	
	private void on_cmdDel_clicked(object sender, EventArgs e){ // remove selected bookmark from list
		userSettingReader.delBookmark(selectedBookmark);
	}		
	
	private void on_cmdZoomIn_clicked(object sender, EventArgs e){ // Zoom In current image
		zoomed++;
		mainImageViewer.FromPixbuf = getImage(selectedImage, 3);
	}		
	
	private void on_cmdZoomOut_clicked(object sender, EventArgs e){ // Zoom out current image
		zoomed--;
		mainImageViewer.FromPixbuf = getImage(selectedImage, 4);
	}		
	
	private void on_cmdRotateLeft_Clicked(object sender, EventArgs e){ // Rotate left current image
		rotated--;
		if(rotated==-3) rotated=1;
		mainImageViewer.FromPixbuf = getImage(selectedImage, 5);
	}
	
	private void on_cmdRotateRight_Clicked(object sender, EventArgs e){ // Rotate right current image
		rotated++;
		if(rotated==3) rotated=-1;
		mainImageViewer.FromPixbuf = getImage(selectedImage, 6);
	}	

	private void on_cmdBackground_clicked(object sender, EventArgs e){ // set current image as wallpaper
		string gconf_args;
		gconf_args = " -t str -s /desktop/gnome/background/picture_filename "; // args to gconftool command
		gconf_args+= selectedImage.completepath;
		try {
			for(int k=0;k<6;k++){
				Process p = new Process();
				p.StartInfo.FileName = "gconftool";
				p.StartInfo.Arguments = gconf_args;
				p.StartInfo.UseShellExecute = true;
				p.StartInfo.CreateNoWindow = true;
				p.Start();
			}
		}
		catch(Exception) {
			Console.WriteLine("ERROR Using gconftool"); // operation not supported
		}		
	}		
	
	private Pixbuf getImage(ckaImage img,int newView){ // retreives the selected image
		Pixbuf res=null;
		double img_height, img_width, newH,newW;		
		Pixbuf tmp = new Gdk.Pixbuf(img.completepath);

		switch(newView) { // how to display image
			case 0: // default view
				res =tmp;
				break;
			case 1: // Original size
				if(!imageRotated) res = (goRotate(tmp)).ScaleSimple(tmp.Width,tmp.Height, Gdk.InterpType.Bilinear);
				else res = (goRotate(tmp)).ScaleSimple(tmp.Height,tmp.Width, Gdk.InterpType.Bilinear);
				break;
			case 2: // Fit to Window
				double win_height = imgViewport.Allocation.Height;
				double win_width = imgViewport.Allocation.Width;
				img_height = img.height;
				img_width = img.width;
				double rH = (double) (win_height / img_height);
				double rW = (double) (win_width / img_width);
				if(rH>rW) { newH = img_height * rW; newW = img_width * rW; }
				else { newH = img_height * rH; newW = img_width * rH; }
				
				if(!imageRotated) res = (goRotate(tmp)).ScaleSimple((int)newW,(int) newH, Gdk.InterpType.Bilinear);	
				else res = (goRotate(tmp)).ScaleSimple((int)newH,(int) newW, Gdk.InterpType.Bilinear);
					
				break;
			case 3: // Zoom In
				img_width = mainImageViewer.Pixbuf.Width;
				img_height = mainImageViewer.Pixbuf.Height;
				newW = img_width+((img_width/zoomFactor)* Math.Abs(zoomed));
				newH = img_height+((img_height/zoomFactor* Math.Abs(zoomed)));
				res =(goRotate(tmp)).ScaleSimple ((int)newW,(int) newH, Gdk.InterpType.Bilinear);
				break;
			case 4:	// Zoom Out
				img_width = mainImageViewer.Pixbuf.Width;
				img_height = mainImageViewer.Pixbuf.Height;
				newW = img_width-((img_width/zoomFactor)* Math.Abs(zoomed));
				newH = img_height-((img_height/zoomFactor)* Math.Abs(zoomed));
				if((newH>10)&&(newW>10)) res = (goRotate(tmp)).ScaleSimple ((int)newW,(int) newH, Gdk.InterpType.Bilinear);
				else res =  mainImageViewer.Pixbuf;
				break;	
			case 5: // Rotate Left
				imageRotated = !(imageRotated);
				res =  mainImageViewer.Pixbuf.RotateSimple(PixbufRotation.Counterclockwise);
				break;
			case 6: // Rotate Right
				imageRotated = !(imageRotated);
				res = mainImageViewer.Pixbuf.RotateSimple(PixbufRotation.Clockwise);
				break;
			case 7: // First time view
				win_height = imgViewport.Allocation.Height;
				win_width = imgViewport.Allocation.Width;
				img_height = img.height;
				img_width = img.width;
				rH = (double) (win_height / img_height);
				rW = (double) (win_width / img_width);
				if(rH>rW) { newH = img_height * rW; newW = img_width * rW; }
				else { newH = img_height * rH; newW = img_width * rH; }
				res = tmp.ScaleSimple((int)newW,(int) newH, Gdk.InterpType.Bilinear);
				break;				
		}
		return res;
	}
		
	private Pixbuf goRotate(Pixbuf src){ // rotate image 
		Pixbuf tmp = null;
		switch(rotated){ // how many times image has being rotated
			case 0:
				tmp = src; break;
			case -2:
				tmp = src.RotateSimple(PixbufRotation.Upsidedown); break;
			case -1:
				tmp = src.RotateSimple(PixbufRotation.Counterclockwise); break;
			case 1:
				tmp = src.RotateSimple(PixbufRotation.Clockwise); break;
			case 2:
				tmp = src.RotateSimple(PixbufRotation.Upsidedown); break;
		}    	
		return tmp;
	}
		
	private void setupCommandButtons(bool status) { // used to enable/disable all the action button
		cmdFitToWin.Sensitive = status;
		cmdOriginalSize.Sensitive = status;
		cmdBackground.Sensitive = status;
		cmdZoomIn.Sensitive = status;
		cmdZoomOut.Sensitive = status;
		cmdRotateLeft.Sensitive = status;
		cmdRotateRight.Sensitive = status;
		cmdPreviousImg.Sensitive = status;
		cmdNextImg.Sensitive = status;
		cmdLastImg.Sensitive = status;
		cmdFirstImg.Sensitive = status;
	}
	
	private void setupThumbsViewer(){ // Setup thumbs treeview
		TreeViewColumn col = new TreeViewColumn();
		CellRendererPixbuf pixr = new CellRendererPixbuf();
		col.AddAttribute(pixr, "pixbuf", 0);  // it will display an image on the left (the image thumb) ....
		col.PackStart(pixr, false);
		col.SetCellDataFunc(pixr, new TreeCellDataFunc(ckaPictureView));
		thumbsViewer.AppendColumn (col);
		
		col = new TreeViewColumn ();
		CellRenderer colr = new CellRendererText();
		col.AddAttribute (colr, "markup", 1);  // ...and a lot of formatted information
		col.PackStart(colr, true); 
		col.SetCellDataFunc(colr, new TreeCellDataFunc(ckaInfoView));
		thumbsViewer.AppendColumn (col);
		
		thumbsViewer.Selection.Changed += onImgSelected; 	
		thumbsStore = new ListStore (typeof (ckaImage));
		thumbsViewer.Model = thumbsStore;	
	}
		
	private void onImgSelected(object o, EventArgs args){ // called when an image is selected on the thumbs treeview
		imageRotated = false;
		rotated = 0;
		zoomed = 0;
        TreeModel model;
        TreeIter iterSelected;
 		if(((TreeSelection)o).GetSelected (out model, out iterSelected)) {
        		ckaImage X = (ckaImage) model.GetValue (iterSelected, 0);
        		try{
        			mainImageViewer.FromPixbuf = getImage(X, typeOfView);
        			selectedImage = X;
        			pushOnVirtualStatusBar(X.completepath + "  (" + X.width + "x" + X.height + ")");
        			imageIsSelected = true;
        			setupCommandButtons(true);
        		}
        		catch(Exception){
        			/* TODO */
        		}
	    }
	}		

	// function called to display image thumbs in the treeview 
	private void ckaPictureView (Gtk.TreeViewColumn tree_column,Gtk.CellRenderer cell, Gtk.TreeModel tree_model,Gtk.TreeIter iter){
		ckaImage nf = (ckaImage) thumbsStore.GetValue(iter,0);
		((CellRendererPixbuf) cell).Pixbuf = nf.getThumb();
	}		
	// function called to display image information in the treeview
	private void ckaInfoView (Gtk.TreeViewColumn tree_column,Gtk.CellRenderer cell, Gtk.TreeModel tree_model,Gtk.TreeIter iter){
		ckaImage nf = (ckaImage) thumbsStore.GetValue(iter,0);
		((CellRendererText) cell).Markup = nf.getPublicView();
	}	
	
	private void OnWindowDeleteEvent (object sender, DeleteEventArgs a){ // event handler for the main window
		userSettingReader.saveBookmarks(false);
		Application.Quit ();
		a.RetVal = true;
	}
		
	private void keyHandler(object sender, KeyReleaseEventArgs a){ // key event handler
		switch(a.Event.Key){ // what key has being presed
			case Gdk.Key.F11: // (Fullscreen)
				if(fullscreen) { mainWindow.Unfullscreen(); fullscreen = false; }
				else { mainWindow.Fullscreen(); fullscreen = true; }
				break;
			case Gdk.Key.m: // M (Maximize)
				if(maximized) { mainWindow.Unmaximize(); maximized = false; }
				else { mainWindow.Maximize(); maximized = true; }				
				break;	
			case Gdk.Key.o: // O
				on_mnuHelp_About_activate(null,null);
				break;
			case Gdk.Key.Escape:
				if(fullscreen) { mainWindow.Unfullscreen(); fullscreen = false; }
				else { on_mnuFile_quit_activate(null,null); }
				break;
			case Gdk.Key.Key_1: 
			case Gdk.Key.KP_1: // 1
				fileSystemBook.Visible = !(fileSystemBook.Visible); 			
				break;
			case Gdk.Key.Key_2:
			case Gdk.Key.KP_2: // 2
				vpaned1.Visible = !(vpaned1.Visible);
				break;
			case Gdk.Key.Key_3:
			case Gdk.Key.KP_3: // 3
				tools.Visible = !(tools.Visible);
				break;
			case Gdk.Key.F1: // F1 (help)
				new ckaHelp(mainWindow);
				break;				
			case Gdk.Key.q: // (Fit to win)
				if(imageIsSelected) on_cmdFitToWin_clicked(null,null);
				break;
			case Gdk.Key.w: // (Original size)
				if(imageIsSelected) on_cmdOriginalSize_clicked(null,null);
				break;
			case Gdk.Key.a: // (Rotate left)
				if(imageIsSelected) on_cmdRotateLeft_Clicked(null,null);
				break;
			case Gdk.Key.s: // (Rotate right)
				if(imageIsSelected) on_cmdRotateRight_Clicked(null,null);
				break;
			case Gdk.Key.minus: // (Zoomi Out)
				if(imageIsSelected) on_cmdZoomOut_clicked(null,null);
				break;
			case Gdk.Key.plus: // (Zoom In)
				if(imageIsSelected) on_cmdZoomIn_clicked(null,null);
				break;
			case Gdk.Key.Page_Up: // (First)
				browseImage(1);
				break;
			case Gdk.Key.Page_Down: // (Last)
				browseImage(4);
				break;
			case Gdk.Key.leftarrow: // (Previous)
				browseImage(2);
				break;
			case Gdk.Key.rightarrow: // (Next)
				browseImage(3);
				break;
		}
	}
	
	private void  on_mnuOptions_aFit_activate(object o, EventArgs args){ // fit to window clicked
		if(typeOfView==0) typeOfView=7;
		else typeOfView=0;
	}

	private void  on_mnuHelp_About_activate(object o, EventArgs args){ // info clicked
		new ckaAbout(mainWindow,ckaVersion);
	}
		
	private void  on_mnuFile_quit_activate(object o, EventArgs args){ // quit clicked
		userSettingReader.saveBookmarks(false);
		Application.Quit ();
	}		

	private void readDirAndPopulateArray(){ // read current folder and add images to the thumbs viewer
		thumbsStore.Clear(); 
		howManyImages = 0; // the counter of the images 
		string[] arrayOfFIle = Directory.GetFiles(folderToRead); // all the files in current folder
		for(long k=0;k<arrayOfFIle.Length;k++){
			if(isImageSupported(arrayOfFIle[k])){ // if current file is a supported image
				ckaImage X = new ckaImage(folderToRead,arrayOfFIle[k],0,thumbsHeight,thumbsWeight); // create a ckaImage
				thumbsStore.AppendValues(X); // add it to the treevidew
				while (Gtk.Global.EventsPending) Gtk.Main.Iteration(); // IMPORTANT: this loop is necessary in order to improve performance
				howManyImages++;
			}
		}	
	}	
		
	private bool isImageSupported(string name){ // determines if the image is supported
		int pos = name.LastIndexOf(".");
		string extension = "#"+name.Substring(pos+1,name.Length-pos-1).ToLower()+"#";
		if(supportedFormat.IndexOf(extension)>-1) return true;
		else return false;
	}		
		
	private void pushOnVirtualStatusBar(string textToPush){ // add the provided text to the status bar
		string message = "<span size=\"smaller\"><b>Image: </b>"+textToPush+"</span>";
		lblStatus1.Text = message;
		lblStatus1.UseMarkup = true;
	}

}

/* ################################################################ */

public class ckaFileSystemReader { // the class will be used in order to read file system content and populate the treeview
    public string currentPath; // path to be readed
    private bool showHid; // show hidden folder ?
    private ListStore store; // the liststore of the file system treeview that will be used to populate it 
    private Gdk.Pixbuf cFolder; // the standard folder icon used in the file system treeview
  	
  	public ckaFileSystemReader(string cp, ListStore ls,bool showHidden) { // main constructor
      this.currentPath = cp; // set the current path (folder to be readed)
      this.store = ls; // set the liststore
      this.showHid = showHidden; // set if the hidden folder will be displayed
	  cFolder = new Pixbuf(null, "ckaFolder.png").ScaleSimple (16, 16, Gdk.InterpType.Bilinear); // get the folder icon
	}

	private void readDir(string path){ // used to read the provided path and to populate the treeview with its content
		string[] dirList = Directory.GetDirectories(path); // get the path
		TreeIter iter = new TreeIter(); 
		string ppath = getParentPath();
		if(ppath!=currentPath) iter = store.AppendValues("..",cFolder,ppath);
		for(int j=0;j<dirList.Length;j++){ // loop through the folder inside the provided path
			string nm = getDirName(dirList[j]); // retreives folder name
			if(showThisDir(nm)){ // id the folder should be displayed...
				iter = store.AppendValues(nm,cFolder,dirList[j]); // ...adds the folder to the treeview
			}
		}
	}
	
	public void refresh(){ // used to refresh the FS TreeView
		store.Clear(); readDir(currentPath);
	}
	
	private string getParentPath(){ // used to retreives the parent folder of current path
		string res = "";
		int pos = currentPath.LastIndexOf("/");
		if(pos<1) res = "/"; // we are at the root folder
		else res = currentPath.Substring(0,pos);
		return res;
	}
		
	private void readDir(string path,TreeIter father){ // called recursively in order to build tree folder structure
		TreeIter child;
		string[] dirList = Directory.GetDirectories(path);
		for(int j=0;j<dirList.Length;j++){
			string nm = getDirName(dirList[j]);
			if(showThisDir(nm)){		
				child = store.AppendValues(father,nm,cFolder,dirList[j]);
				readDir(dirList[j],child);
			}
		}
	}
	
	private string getDirName(string fullName){ // return the name of the folder
		int pos = fullName.LastIndexOf("/");
		string dirName = fullName.Substring(pos+1,fullName.Length-pos-1);
		return dirName;
	}
	
	private bool showThisDir(string dirName){ // if the folder is an hidden folder decidesif it should be displayed
		bool isHidden;
		if(dirName.Substring(0,1).Equals(".") && !dirName.Substring(0,2).Equals("..")) isHidden = true;
		else isHidden = false;
		if(isHidden==showHid) return true;
		else return false;
	}	
}

/* ################################################################ */

public class ckaImage{ // this class represents an image object
	public string name; // the name of the image (filename)
	public string	fullpath; // the path to the folder containing the image
	public string completepath; // the complete path (full+name)
	public long width=0; // the image's width
	public long height=0; // the image's heigth
	public long size; // the size (kb) of the image
	public bool hasThumb; // used to retreives image thumb for the thumbs viewer
	public int HSize; // thumb height
	public int WSize; // thumb width
	public Pixbuf thumb; // the thumbnail 
	
	public ckaImage(string pathTo,string imgname, long imgsize, int hsize, int wsize){ // main constructor
		int pos;
		fullpath = pathTo;
		completepath = imgname;
		size = imgsize;
		pos = completepath.LastIndexOf("/");
		name = completepath.Substring(pos+1,completepath.Length-pos-1);
		HSize = hsize;
		WSize = wsize;
		hasThumb = false;
		go();
	}
	
	private void go(){ // this method is used in order to improve thumbs generation process
		Thread t = new Thread(new ThreadStart(generateThumb)); // every thumb is generated by a thread
		t.Start();		
	}	
	
	public string getPublicView(){ // this method returns a string containing image informations in a defined format
		string res="";
		res+= "<b>"+name+"</b>\n<span size=\"smaller\">"; 
		if(width>0) res+=width+"x"+height+"</span>\n"+ getSize();
		else res+="Info not available</span>\n21200Kb";
		return res;
	}
	
	private string getSize(){ // the method is used to transform the size from byte to kb;
		if(size<1000) return size + " byte";
		else if(size>=1000) return Math.Abs(size/1000) + " Kb";
		else return "Unknown";
	}
	
	private void generateThumb(){ // this method generate the thumb for the image and store it in the local object
		try{
			FileInfo x = new FileInfo(completepath); // read the file
			size = x.Length; // get filesize
			x = null; // close the fileinfo object in order to improve performance
			Pixbuf tmp = new Pixbuf(completepath); // create the Pixbuf with the provided filename
			width = tmp.Width; // set the width properties
			height = tmp.Height; // set the height properties
			thumb = tmp.ScaleSimple (WSize, HSize, Gdk.InterpType.Bilinear); // scale the image to the provided size 		
		} 
		catch(Exception){ // if the previous code block fails, then a NA thumbs will be returned
			thumb = new Pixbuf(null, "na.png").ScaleSimple (WSize, HSize, Gdk.InterpType.Bilinear);
		}
		hasThumb = true; // the thumbs has being generated (remembre that we are working with thread)
	}
	
	public Pixbuf getThumb(){ // public method used to retreives the image's thumb 
		Pixbuf res;
		if(hasThumb==true) res = thumb;
		else res = new Pixbuf(null, "na.png").ScaleSimple (WSize, HSize, Gdk.InterpType.Bilinear);
		return res;	
	}
}

/* ################################################################ */

public class settingReader{ // Objects of this class will be used to read application settings in XML file
	private ListStore store;
	private ArrayList bkmrkList;
	private string filename;
	
	public settingReader(ListStore bkmrkStore){ // main constructor
		store = bkmrkStore; // the Liststore passe dto the ctor is the ListStore of the Bookmark TreeView of the Main class
		bkmrkList = new ArrayList(); // define an array of object that will be populated with the stored bookmark
		filename = System.Environment.GetEnvironmentVariable("HOME")+"/.cooka"; // load standard file in the user home folder
	}

	public void readAndPopulateBookmark(){ // read the settings file and populates the arraylist 
		XmlDocument xdsettings = new XmlDocument(); // the documnet that will be readed
 		try{
 			xdsettings.Load(filename); // load the file
 			XmlTextReader xr = new XmlTextReader(filename); // create a XmlTextREader in order to read file's content
			while(xr.Read()){  // start the main loop through the bookmark nodes of the file 
				if(xr.NodeType == XmlNodeType.Element) {
					if(xr.Name=="BOOKMARK"){ // if the element name is BOOKMARK then
						while(xr.MoveToNextAttribute()){ // read all the attributes and..
							if(xr.Name=="URL") bkmrkList.Add(xr.Value); // ...add them to the arraylist
						}
					}
				}
			}
			xr.Close(); // close the XmlTextReader
 		}
 		catch(Exception){ // this section will be used all the times the file is not present (first Cooka run)
			saveBookmarks(true); // if there was an error, then the application save the bookmark creating the new setting file
 		}	
 		populateBookmarkViewer(); // finally, the array list of bookmark is used to add items to the bookmark TreeView
	}

	public void saveBookmarks(bool firsttime){ // Save the bookmarks to the user's setting file
		XmlDocument xdsettings = new XmlDocument(); 
 		XmlElement xe = xdsettings.CreateElement("COOKA-BKMRKS"); // The root element of the xml settings file
 		xdsettings.AppendChild(xe); // append the root element
 		if(!firsttime) { // if this is not the first time, then add to the file all the available bookmarks (if present)
			IEnumerator myEnumerator = bkmrkList.GetEnumerator(); // loop through the ArrayList
			while(myEnumerator.MoveNext()){ // read current item...
				XmlElement xpe = xdsettings.CreateElement("BOOKMARK"); // ...create an XML element...
	 			xpe.SetAttribute("URL",(string)myEnumerator.Current); // ...set the element attribute URL to the bookmark...
	 			xe.AppendChild(xpe); // ...and finally append the bookmark to the root node
	 		}
 		}
 		xdsettings.Save(filename);	// At the end, the file will be saved. If it's the first time the file will be empty
	}

	public void addBookmark(string newBkmrk){ // the method is called from outside the class in order to add new bookmark
		bkmrkList.Add(newBkmrk); // the new bookmark is added to the ArrayList
		populateBookmarkViewer(); // the Bookmark Treeview is re-populated
	}
	
	public void delBookmark(string delBkmrk){ //  the method is called from outside the class in order to remove bookmark
		store.Clear(); // clear the bookmark treeview
		IEnumerator myEnumerator = bkmrkList.GetEnumerator(); // loop through available bookmark
		while(myEnumerator.MoveNext()){ 
			if (delBkmrk.Equals((string)myEnumerator.Current)) { // if current bookmark match the selected item...
				bkmrkList.Remove(  (string)myEnumerator.Current); // ..the item is removed from the ArrayList
				break;
			}
		}		
		populateBookmarkViewer(); // And the Bookmark treeview is refreshed
	}
	
	private void populateBookmarkViewer(){ // used to add ArrayList items to the Bookmark treeview
		store.Clear(); // clear the treeview
		IEnumerator myEnumerator = bkmrkList.GetEnumerator(); // retreives an iterator for the arraylist
		while(myEnumerator.MoveNext()){ // loop through items
			store.AppendValues((string)myEnumerator.Current); // add items to the treeview
		}
	}
}

/* ################################################################ */

public class ckaAbout {  // The class that implements the About window
	[Widget] Gtk.Window aboutWin;
	[Widget] Gtk.Image imgLogo;
	[Widget] Gtk.Label lblVersion;
	[Widget] Gtk.Button cmdClose;
	private Gtk.Window own;
	private string ckaVersion;
	
	public ckaAbout(Gtk.Window owner,string version){ // Main constructor 
		Glade.XML gxml = new Glade.XML (null, "gui.glade", "aboutWin", null); // Connect to the Glade resource
		gxml.Autoconnect (this);
		own = owner; // set the owner of the About window (it's the main application window)
		ckaVersion = version; // set the current version in order to display it on the about window
		setupGui(); // draw the user interface of the window
	}
	
	private void setupGui(){ // Used to set up many GUI properties
		aboutWin.Icon = new Gdk.Pixbuf(null,"logo_v1_ico.png"); // the Cooka icon
		aboutWin.Title = "Cooka"; // the title of the window
		aboutWin.Resizable = false; // not resizable
		aboutWin.TransientFor = own; // make the window a modal window for the main applciation window
		aboutWin.SetPosition(Gtk.WindowPosition.CenterOnParent); // set the window position
		aboutWin.Modal = true; 
		imgLogo.FromPixbuf = new Gdk.Pixbuf(null,"logo_v1_about.png"); // set the logo image source
		cmdClose.Clicked+=on_quit; // close button event handler
		lblVersion.Text = "Release "+ckaVersion; // set the release label text
		aboutWin.ShowAll(); // display the window
	}
	
	private void  on_quit(object o, EventArgs args){ aboutWin.Destroy();} // the delegate 
	
}

/* ################################################################ */

public class ckaHelp { // the class that implements the Help window (almost all the properties was defined in Glade)
	[Widget] Gtk.Window helpWin;
	private Gtk.Window own;
	
	public ckaHelp(Gtk.Window owner){ // main constructor
		Glade.XML gxml = new Glade.XML (null, "gui.glade", "helpWin", null); // connect to the glade resource
		gxml.Autoconnect (this); 
		own = owner; // set the owner of this modal window
		setupGui(); // 
	}

	private void setupGui(){ // Used to set up many GUI properties
		helpWin.Icon = new Gdk.Pixbuf(null,"logo_v1_ico.png"); // window's icon
		helpWin.Title = "Help"; // the main title
		helpWin.Resizable = false; // not resizable
		helpWin.TransientFor = own; // make the window a modal window for the main applciation window
		helpWin.SetPosition(Gtk.WindowPosition.CenterAlways); // set the window position
		helpWin.Modal = true;
		helpWin.ShowAll(); // display the window
	}
}